/*
Copyright (C) 2005-2009 Niels Ott

Licensed under the Apache License, Version 2.0 (the "License"); you may not use 
this file except in compliance with the License. You may obtain a copy of the 
License at 
     http://www.apache.org/licenses/LICENSE-2.0 
Unless required by applicable law or agreed to in writing, software distributed 
under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
CONDITIONS OF ANY KIND, either express or implied. See the License for the specific
language governing permissions and limitations under the License.
 */

package de.drni.bananasplit;

import de.drni.bananasplit.affix.Affix;
import de.drni.bananasplit.affix.BoundingSuffixCollection;
import de.drni.bananasplit.affix.InflectionSuffixCollection;
import de.drni.bananasplit.affix.UmlautInfixCollection;
import de.drni.bananasplit.omat.InfixOMat;
import de.drni.bananasplit.omat.LookupAffixOMat;
import de.drni.bananasplit.omat.LookupTwostepAffixOMat;
import de.drni.bananasplit.omat.SuffixOMat;
import de.drni.bananasplit.simpledict.SimpleDictEntry;
import de.drni.bananasplit.simpledict.SimpleDictionaryInterface;


/**
This is the standalone version of BananaSplit that operates
on simple word lists resp. dictionaries. 
Take a look at the BananaSplitTester source code
for a reference implementation. 
Like in earlier versions, one can use BananaSplit as a
dictionary resource. In this case, use the 
de.drni.bananasplit.gn package in addition.
For further information, please check the 
<a href="http://www.drni.de/niels/cl/BananaSplit/" target="_blank">home page of BananaSplit</a>.
<br>&nbsp;<br>
A compound splitter that uses a dictionary resource.
The basic assumption is that compounds consist of two parts only.
Each of these Atoms (non-dividable parts) is looked up in the dictionary,
with consideration of bounding suffixation. It is not important
that one of the Atoms (or both) may be a compound itsself.
Even though you get the Atoms as WordSenses, be aware that
the first word sense is taken only. 
The first hit from the dictionary will be used for operation,
no matter if the dictionary holds several entries with
distinct parts-of-speach or the like.
@author <a href="http://www.drni.de/niels/">Niels Ott</a>
@version $Id: BananaSplit.java 37 2009-02-05 15:26:03Z niels $
*/
public class BananaSplit {

private SimpleDictionaryInterface myDict;
private String myCompound = null;
private SimpleDictEntry myLeftAtom = null;
private Affix mySuffix = null;
private Affix myInflection = null;
private Affix myInfix = null;
private SimpleDictEntry myRightAtom = null;


	/**
	Constructor
	@param  dict dictionary/word list to use.
	*/
	public BananaSplit(SimpleDictionaryInterface dict) {
	
		myDict = dict;
	
	}

	/**
	Splits a compound into its left and right atom and 
	several affixes. The affixes may be a bounding suffix, 
	umlaut infix and inflection suffix. You can retrieved the result
	with the methods described below.
	@param compound The compound to split.
	@throws Exception if the GermaNet lookup results into something completely nonsensical.
	(Should never happen, but...)
	@return -1 if the compound cannot be split, 0 in case if success and 1 if the compound is found as a whole word in GermaNet.
	*/
	public int splitCompound(String compound) throws Exception {
	
		myCompound = compound;
		myLeftAtom = null;
		myRightAtom = null;
		mySuffix = null;
		myInfix = null;

		// go from right to left with the compound
		// don't stop before as much as possible is consumed... (longest match)
		// try this for ALL possible inflections

		String lookupword = "";
		SimpleDictEntry rightmatch = null;

		LookupTwostepAffixOMat inf_omat = new LookupTwostepAffixOMat(myDict,
			new InfixOMat(new UmlautInfixCollection()),
			new SuffixOMat(new InflectionSuffixCollection()) );
		
		// produce words from right to left
		for (int i = myCompound.length()-1; i >= 0;  i-- ) {
		
			lookupword = "" + myCompound.charAt(i) + lookupword;
			
			// FIND IT
			SimpleDictEntry match = inf_omat.MEDfind(lookupword);
			
			// do we have a match?
			if ( match != null ) {
				rightmatch = match;
				myInfix = inf_omat.getAppliedAffix(0);
				myInflection = inf_omat.getAppliedAffix(1);
			}
		}

		
  		// nothing matched anyways, fail!
  		if (rightmatch == null ) {
  			return -1;
  		}
		
		
		// length of the remaining left atom
		// - but consider the inflection business!
		int left_len = myCompound.length() - rightmatch.getWord().length()
				+ myInflection.getLengthChange();
				
		// now if the left part is empty/consumed, we got it already
		if (left_len == 0) {
			myRightAtom = rightmatch;
			return 1;
		}		
		
		// do a sanity check for rare cases in which GermaNet API gives
		// completely weird responses that may lead to further trouble
	  	if  ( left_len < 0 ) {
	  		throw( new Exception("Dictionary/Word List lookup inconsistency: Queried for \""+
	  			myCompound + "\", found \"" + rightmatch + "\".") );
	   	
  	  	}
		
		
		// ok so only the right atom was found, now let's see for 
		// the left atom!
		String left = myCompound.substring(0, left_len);
		SimpleDictEntry leftmatch = null;		
				
		// now apply all kinds of "de-compound-suffixation" on the left part
		// and check for each result		
		LookupAffixOMat bound_omat = new LookupAffixOMat(myDict, 
			new SuffixOMat(new BoundingSuffixCollection()));
		
		// FIND IT
		leftmatch = bound_omat.find(left);
		
		// if the left atom worked, we are done
		if (leftmatch != null) {
			myLeftAtom = leftmatch;
			myRightAtom = rightmatch;
			mySuffix = bound_omat.getAppliedAffix();
			
			return 0;
		}
			
	
		return -1;
	}
	
	/**
	Get the left atom after splitting.
	@return the left atom of the last compound splitting action or null if there
		was no such action or the action failed. Returns the first word sense only.
	*/	
	private SimpleDictEntry getLeftAtom() {

		return myLeftAtom;
	
	}

	
	/**
	Get the right atom after splitting.
	@return the right atom of the last compound splitting action or null if there
		was no such action or the action failed.  Returns the first word sense only.
	*/	
	private SimpleDictEntry getRightAtom() {
	
		return myRightAtom;
		
	}
	
	/**
	Get the the compound we're working on as plain text.
	@return compound of the last compound splitting action or null if there
		was no such action. 
	*/	
	private String getPlainCompound() {
	
		return myCompound;
		
	}

	/**
	Get the the compound we're working on as Compound data structure.
	@return compound of the last compound splitting action or null if there
		was no such action. 
	*/	
	public Compound getCompound() {
	
		return new Compound(getLeftAtom(), getRightAtom(),
			mySuffix, myInflection, myInfix, getPlainCompound());
		
	}
	
	/**
		Present a human-readable analysis of the Atoms and affixes 
		(umlaut infix, bounding suffix, inflection suffix) of a compound.
		@param preamble Stuff to put in front of the analysis. Use e.g. \Tree for LaTeX Qtree
		@param commentsign Stuff to put inf front of the comment. Use e.g. % for LaTeX comment.
	*/
	public String toString	(String preamble, String commentsign) {
	
		Compound CompObj = getCompound();
		
		//return "" + CompObj;
	
		// seems we had a failure
		if ( ( getLeftAtom() == null) &&  ( getRightAtom() == null) ) {
			return preamble + CompObj 
				+ commentsign + "No analysis possible.";
		}
		
		// the whole word was present in GermaNet... so why bother
		if ( ( getLeftAtom() == null) ) {
			return preamble + CompObj 
				+ commentsign + "Whole word was present, no splitting required.";
		}
		
		// full splitting analysis, yeah!
		return preamble + CompObj 
			 + commentsign + "Analysis based on two Atoms.";
	
	}
	
	/**
		Present a human-readable analysis of the Atoms and affixes 
		(umlaut infix, bounding suffix, inflection suffix) of a compound.
		Default version.
	*/
	public String toString() {
		
		return toString("","\n# ");
	}
	
	


}
