/*
Copyright (C) 2005-2009 Niels Ott

Licensed under the Apache License, Version 2.0 (the "License"); you may not use 
this file except in compliance with the License. You may obtain a copy of the 
License at 
     http://www.apache.org/licenses/LICENSE-2.0 
Unless required by applicable law or agreed to in writing, software distributed 
under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
CONDITIONS OF ANY KIND, either express or implied. See the License for the specific
language governing permissions and limitations under the License.
 */

package de.drni.bananasplit;

import de.drni.bananasplit.affix.Affix;
import de.drni.bananasplit.simpledict.SimpleDictEntry;


/**
A simple class holding a compound
that consists of two parts (Atoms) and
several affixes. (Umlaut infix, bounding suffix, inflectional suffix.)
If the compound is not split, the left Atom is null.
@author <a href="http://www.drni.de/niels/">Niels Ott</a>
@version $Id: Compound.java 37 2009-02-05 15:26:03Z niels $
*/
public class Compound {

private String left;
private String left_pos;
private String right;
private String right_pos;
private Affix myBoundingSuffix;
private Affix myInflectionSuffix;
private Affix myInfix;
private String myCompound;

	/**
	Constructor.
	@param Left the left Atom of the compound
	@param Right the right Atom of the compound
	@param BoundSuffix the bounding suffix in between
	@param InflectionSuffix inflectional suffix at the end.
	@param Infix some infix, most probably for umlauting.
	@param Comp the plain Compound 
	*/
	public Compound(SimpleDictEntry Left, SimpleDictEntry Right, 
	   Affix BoundSuffix, Affix InflectionSuffix, Affix Infix, String Comp) {
	   
	   	if ( Left == null ) {
			left = null;
			left_pos = null;
		} else {
	   		left = Left.getWord();
			left_pos = "" + Left.getPartOfSpeech();
			left_pos = left_pos.toUpperCase();
		}
		
		if ( Right == null ) {	
			right = null;
			right_pos = null;
		} else {	
			right = Right.getWord();
			right_pos = "" + Right.getPartOfSpeech();
			right_pos = right_pos.toUpperCase();
		}	
		
		myBoundingSuffix = BoundSuffix;
		myInflectionSuffix = InflectionSuffix;
		myInfix = Infix;
		myCompound = Comp;
	   
	}
	
	
	/**
	@return the left atom of the compound. May be null if the compound has not been split
	*/
	public String getLeftAtom() {
		return left;
	}

	/**
	@return the left atom of the compound, but return the right Atom if the left one is
	empty.
	*/
	public String getLeftAtomSafely() {
		if ( getLeftAtom() == null) {
			return getRightAtom();
		}
		
		return getLeftAtom();
	}

	/**
	@return the right atom of the compound, is filled unless something went wrong, then null
	*/
	public String getRightAtom() {
		return right;
	}

	/**
	@return the POS of the left atom of the compound. May be null if the compound has not been split
	*/
	public String getLeftPOS() {
		return left_pos;
	}

	/**
	@return the POS of left atom of the compound, but return the POS of the right Atom if the left one is
	empty.
	*/
	public String getLeftPOSSafely() {
		if ( getLeftPOS() == null) {
			return getRightPOS();
		}
		
		return getLeftPOS();
	}

	/**
	@return the POS of the right atom of the compound, is filled unless something went wrong, then null
	*/
	public String getRightPOS() {
		return right_pos;
	}


	/**
	@return the bounding suffix of the compound
	*/
	public Affix getBoundingSuffix() {
		return myBoundingSuffix;
	}

	/**
	@return the inflectional suffix of the compound
	*/
	public Affix getInflectionSuffix() {
		return myInflectionSuffix;
	}

	/**
	@return the infix of the compound
	*/
	public Affix getInfix() {
		return myInfix;
	}

	/**
	 * @deprecated use getLemma() instead.
	 * @return returns the lemma of the entire compound, this name is
	 * misleading.
	 */
	public String getWordStem() {

		return getLemma();
		
	}

	/**
	 * @return returns the lemma of the entire compund.
	 */
	public String getLemma() {

		// left atom empty, then the stem is just the right atom
		if (getLeftAtom() == null) {
			return getRightAtom();
		}
		
		// otherwise we need to re-combine the left atom
		String l = getLeftAtom();
		
		// no need to delete?
		if ( myBoundingSuffix.getDel().compareTo("") != 0 ) {
			l = l.substring(0, 
					l.length() - myBoundingSuffix.getDel().length()  );	
			
		}
		
		// add the bounding suffix
		l = l + myBoundingSuffix.getAdd();
		
		// return the whole thing
		return  l + getRightAtom();
		
	}

	
	/**
	@return returns a bracketing analysis of the compound
	*/
	public String toString() {

		// seems we had a failure
		if ( ( getLeftAtom() == null) &&  ( getRightAtom() == null) ) {
			return "[.X " + getPlainCompound() + " ]";
		}
		
		// the whole word was present in GermaNet... so why bother
		if ( ( getLeftAtom() == null) ) {
			return "[." + right_pos + 
			        " [." + right_pos + " " + getRightAtom() + " ] " + 
				" [.U " + getInfix() + " ] " + 
			        " [.I "  + getInflectionSuffix() + " ] " +
				"]";
		}
		
		return "[." + right_pos + 
			" [." + left_pos + " " + getLeftAtom() + " ] " +
			" [.B "  + getBoundingSuffix() + " ] " +
			" [." + right_pos + " " + getRightAtom() + " ] " +
			" [.U " + getInfix() + " ] " + 
			" [.I "  + getInflectionSuffix() + " ] " +
			"]";
		
	}

	/**
	@return returns the plain compound
	*/
	public String getPlainCompound() {

		return myCompound;
		
	}
	
	/**
	@return returns the length of the plain compound
	*/
	public int length() {

		return getPlainCompound().length();
		
	}
	

}
