/*
Copyright (C) 2005-2009 Niels Ott

Licensed under the Apache License, Version 2.0 (the "License"); you may not use 
this file except in compliance with the License. You may obtain a copy of the 
License at 
     http://www.apache.org/licenses/LICENSE-2.0 
Unless required by applicable law or agreed to in writing, software distributed 
under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
CONDITIONS OF ANY KIND, either express or implied. See the License for the specific
language governing permissions and limitations under the License.
 */
package de.drni.bananasplit.omat;

import de.drni.bananasplit.affix.Affix;
import de.drni.bananasplit.simpledict.SimpleDictEntry;
import de.drni.bananasplit.simpledict.SimpleDictionaryInterface;


/**
A class that restores base forms of words which have
suffered from bounding suffixation or inflection or any
kind of affixation
The result is checked for pausibility by using a dictionary resp. word list.
Based on Langer(1998).
@author <a href="http://www.drni.de/niels/">Niels Ott</a>
@version $Id: LookupAffixOMat.java 37 2009-02-05 15:26:03Z niels $
*/

public class LookupAffixOMat {

private SimpleDictionaryInterface myDict;
private AffixOMat myOMAT;
private Affix myAffix;


	/** 
	Constructor
	@param dict the dictionary/word list to use.
	@param omat a affix thingie to use.
	*/
	public LookupAffixOMat(SimpleDictionaryInterface dict, AffixOMat omat) {
	
		myDict = dict;
		myOMAT = omat;
		myAffix = null;
	
	}
	
	
	/**
	Finds the most plausible dictionary/word list entry for a given SuffixOMat.
	Returns the first match the SuffixOMat finds in its list. (Oder
	does matter!)
	@param word the word to look for, applying (de)suffixation
	@return the first first word sense matching or null if no matches.
	*/
	public SimpleDictEntry find(String word) {
	
		String test = null;
		SimpleDictEntry result = null;
		SimpleDictEntry match = null;
		myAffix = null;
		
		myOMAT.init(word);
		
		while (  (test = myOMAT.applyNextAffix() ) != null )  {

			// look up this one in the dictionary
			match = myDict.findWordNoCase(test);	
			
			//System.err.println("find(): test=" + test);
			
			
			// we have a hit!
			if (match != null) {
				result = match;
				myAffix = myOMAT.lastAppliedAffix();
				
				return result;
	
			}

		}
		
		return null;
		
	}

	/**
	Finds the most plausible dictionary/word list entry for a given SuffixOMat.
	Does not care about order of the suffix list but uses
	minimum edit distance.
	@param word the word to look for, applying (de)suffixation
	@return the first word sense matching with minimum edit distance or null if no matches.
	*/
	public SimpleDictEntry MEDfind(String word) {
	
		String test = null;
		float MED = Float.MAX_VALUE; // a high value
		SimpleDictEntry match = null;
		SimpleDictEntry result = null;
		myAffix = null;
		
		myOMAT.init(word);
		
		while (  (test = myOMAT.applyNextAffix() ) != null )  {
		
			// look up this one in the dictionary
			match = myDict.findWordNoCase(test);	
			
			//System.err.println("find(): test=" + test);
			
			
			// we have a hit!
			if (match != null) {
				// if the MED is smaller, keep this one
				if ( myOMAT.lastAppliedAffix().getEditDistance() < MED ) {
					result = match;
					myAffix = myOMAT.lastAppliedAffix();
					MED = myAffix.getEditDistance();
				}
			}

		}
		
		
		return result;
		
	}

	
	/**
	Gets the suffix that came along with the last successful lookup.
	@return the suffix or null if there was no successful lookup
	*/
	public Affix getAppliedAffix() {
	
		return myAffix;
	
	}
	
	

}
