/*
Copyright (C) 2005-2009 Niels Ott

Licensed under the Apache License, Version 2.0 (the "License"); you may not use 
this file except in compliance with the License. You may obtain a copy of the 
License at 
     http://www.apache.org/licenses/LICENSE-2.0 
Unless required by applicable law or agreed to in writing, software distributed 
under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
CONDITIONS OF ANY KIND, either express or implied. See the License for the specific
language governing permissions and limitations under the License.
 */
package de.drni.bananasplit.omat;

import de.drni.bananasplit.affix.Affix;
import de.drni.bananasplit.simpledict.SimpleDictEntry;
import de.drni.bananasplit.simpledict.SimpleDictionaryInterface;

/**
A class that restores base forms of words which have
suffered from bounding suffixation or inflection or any
kind of affixation. In addition
to its brother, LookupAffixOMat, this one can do a two step
procedure, e.g. "repair" umlauting first and then do
"deflection", depending on the AffixOMats you hand over.
The result is checked for pausibility by using the dictionary.
Based on Langer(1998).
@author <a href="http://www.drni.de/niels/">Niels Ott</a>
@version $Id: LookupTwostepAffixOMat.java 37 2009-02-05 15:26:03Z niels $
*/

public class LookupTwostepAffixOMat {

private Affix myAffix1;
private Affix myAffix2;
private AffixOMat myOMAT1;
private LookupAffixOMat myLOMAT2;



	/** 
	Constructor
	@param dict the dictionary/word list to use.
	@param step1omat a affix thingie to use as first step. May fail without consequences.
	@param step2omat a affix thingie to use as second step. Failure leads to feailure.
	*/
	public LookupTwostepAffixOMat(SimpleDictionaryInterface dict, AffixOMat step1omat, AffixOMat step2omat) {
	
		myOMAT1 = step1omat;
		myLOMAT2 = new LookupAffixOMat(dict, step2omat);
		

		myAffix1 = null;
		myAffix2 = null;
	
	}
	
	
	/**
	Finds the most plausible dictionary entry for the SuffixOMats.
	Returns the first match the SuffixOMat finds in its list. (Order
	does matter!)
	@param word the word to look for, applying both SuffixOMats
	@return the first first word sense matching or null if no matches.
	*/
	public SimpleDictEntry find(String word) {
	
		String test = null;
		SimpleDictEntry result = null;
		myAffix1 = null;
		myAffix2 = null;
		
		// try second step alone first
		result = myLOMAT2.find(word);
		
		if (result != null) {
			myAffix1 = new Affix();
			myAffix2 = myLOMAT2.getAppliedAffix();
			return result;
		}
		
		// ok let's try the first step - use the first match
		myOMAT1.init(word);
		test = myOMAT1.applyNextAffix();
		
		// no application of "de-affixation", so give up...
		if (test == null) {
			return null;
		}
		
		// ok if we're here, the step1 AffixOMat did something
		// let's do step 2 again!
		result = myLOMAT2.find(test);
		
		if (result != null) {
			myAffix1 = myOMAT1.lastAppliedAffix();
			myAffix2 = myLOMAT2.getAppliedAffix();
			return result;
		}
		
		// nothing worked out
		return null;
		
		
	}

	/**
	Finds the most plausible dictionary entry entry for a given SuffixOMat.
	Does not care about order of the suffix list but uses
	minimum edit distance.
	@param word the word to look for, applying both AffixOMats
	@return the word sense matching with minimum edit distance or null if no matches.
	*/
	public SimpleDictEntry MEDfind(String word) {
	

		String test = null;
		SimpleDictEntry result = null;
		myAffix1 = null;
		myAffix2 = null;
		float MED = Float.MAX_VALUE; // a high value		
		
		// try second step alone first
		result = myLOMAT2.MEDfind(word);
		
		if (result != null) {
			myAffix1 = new Affix();
			myAffix2 = myLOMAT2.getAppliedAffix();
			return result;
		}
		
		// ok let's try the first step - find the match
		// with shortest MED
		myOMAT1.init(word);
		String medtest = null;
		while (  (medtest = myOMAT1.applyNextAffix() ) != null )  {
		
			if (medtest != null) {
				if ( myOMAT1.lastAppliedAffix().getEditDistance() < MED ) {
					test = medtest;
					MED = myOMAT1.lastAppliedAffix().getEditDistance();
				}
			}

		}		
		
		// no application of "de-affixation", so give up...
		if (test == null) {
			return null;
		}
		
		// ok if we're here, the step1 AffixOMat did something
		// let's do step 2 again!
		result = myLOMAT2.MEDfind(test);
		
		if (result != null) {
			myAffix1 = myOMAT1.lastAppliedAffix();
			myAffix2 = myLOMAT2.getAppliedAffix();
			return result;
		}
		
		// nothing worked out
		return null;
		
	}

	
	/**
	Gets the suffix that came along with the last successful lookup.
	@param whichaffix Which affix to return? For first or second step of processing. (0 or 1)
	@return the suffix or null if there was no successful lookup or wrong number	given.
	*/
	public Affix getAppliedAffix(int whichaffix) {
	
		if (whichaffix == 0 ) {
			return myAffix1;
		}
		if (whichaffix == 1 ) {
			return myAffix2;
		}
		
		return null;
	
	}
	
	

}
