/*
Copyright (C) 2005-2009 Niels Ott

Licensed under the Apache License, Version 2.0 (the "License"); you may not use 
this file except in compliance with the License. You may obtain a copy of the 
License at 
     http://www.apache.org/licenses/LICENSE-2.0 
Unless required by applicable law or agreed to in writing, software distributed 
under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
CONDITIONS OF ANY KIND, either express or implied. See the License for the specific
language governing permissions and limitations under the License.
 */
package de.drni.bananasplit.omat;

import java.util.Iterator;

import de.drni.bananasplit.affix.Affix;

/**
A class that restores base forms of words which have
suffered from bounding suffixation or inflection.
Based on Langer(1998).
@author <a href="http://www.drni.de/niels/">Niels Ott</a>
@version $Id: SuffixOMat.java 37 2009-02-05 15:26:03Z niels $
*/
public class SuffixOMat implements AffixOMat {


private String myWord;
private Iterable<Affix> mySuffixes;
private Iterator<Affix> mySuffixesIterator;
private Affix appliedSuffix;

	/**
	Constructor.
	@param sfx any Iterable containing Affixes that can by
	applied by this Omat. Affixes are of course supposed
	to be suffixes.
	*/
	public SuffixOMat(Iterable<Affix> sfx) {
	
		// operate on lowercase only, reset position
		mySuffixes = sfx;
		
		init(null);
		
	}
	
	/**
		Initialize the SuffoxOMat with a new word to used.
		Resets everything.
		@param word The new word to use from now on;
	*/	
	public void init(String word) {
	
		appliedSuffix = null;
		myWord = word;
		mySuffixesIterator = mySuffixes.iterator();
		
	}
	

	/**
	Takes the word that "suffered" from a (bounding or inflectional) suffixation
	and finds the suggested base form by applying the next appropriate suffixation rule
	@return the suggested base form of the word or null
		if end of list reached or no results.
	*/
	public String applyNextAffix() {
	
		// if empty (no init() run)
		if (myWord == null) {
			return null;
		}


		// end reached?!		
		if ( ! mySuffixesIterator.hasNext() ) {
			return null;
		}
		
		// get the next suffix to work on;	
		Affix suffix = mySuffixesIterator.next();
		
				
		String word = myWord;
		
		// does the current suffix fit
		// (could it have been added somewhen?)
		if ( suffix.getAdd().compareTo("") != 0 ) {
		
			// does deletion match?
			// if so go for it, if not, try next
			if ( word.endsWith(suffix.getAdd()) ) {
			
				word = word.substring(0,
					word.length() - suffix.getAdd().length() );
	
			} else {
				return applyNextAffix();
			}
		
		
		}
		
		// Add the corresponding suffix that
		// is presumed to having been deleted
		word = word + suffix.getDel();
		
		appliedSuffix = suffix;
		
		return word;
	
	}
	
	/**
	Gives you the suffix applied in the last applyNextAffix()
	@return the suffix applied in the last applyNextAffix()
		or null if the end was reached already or no suffix could be applied.
	*/
	public Affix lastAppliedAffix() {
	
		return appliedSuffix;
	
	}



}
