/*
Copyright (C) 2005-2009 Niels Ott

Licensed under the Apache License, Version 2.0 (the "License"); you may not use 
this file except in compliance with the License. You may obtain a copy of the 
License at 
     http://www.apache.org/licenses/LICENSE-2.0 
Unless required by applicable law or agreed to in writing, software distributed 
under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
CONDITIONS OF ANY KIND, either express or implied. See the License for the specific
language governing permissions and limitations under the License.
 */
package de.drni.bananasplit.simpledict;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;

/**
 * A simple dictionary that reads vertical data from a stream
 * or file.
 * The file format consists of one line per entry, with tabulator 
 * separated collumns. The first collumn contains the actual
 * word, the second one the part-of-speech (POS). Further collumns
 * are ignored. Input containing only a word per line is also
 * allowed, in this case all POS will be null.
 * Input lines starting with a hash sign (#) are ignored.
 * @author <a href="http://www.drni.de/niels/">Niels Ott</a>
 * @version $Id: VerticalSimpleDictionary.java 37 2009-02-05 15:26:03Z niels $
 */
public class VerticalSimpleDictionary implements SimpleDictionaryInterface {
	
	private HashMap<String,String> MyLCWords = null;
	private HashMap<String,String> MyWords = null;
	private boolean UseSimplePOS = false;
	
	
	/**
	 * Construct a new dictionary from a file
	 * @param file the path of the file to read from
	 * @throws IOException 
	 */
	public VerticalSimpleDictionary(String file) throws IOException {
		this(file, false);
	}

	/**
	 * Construct a new dictionary from a file
	 * @param file the path of the file to read from
	 * @param simplePOS if set to true, only the first character of the POS column will be used.
	 * 
	 * @throws IOException 
	 */
	public VerticalSimpleDictionary(String file, boolean simplePOS) throws IOException {
		this(new FileInputStream(file), simplePOS);
	}

	/**
	 * Construct a new dictionary from a stream
	 * @param stream the stream to read the data from.
	 * @throws IOException 
	 */
	public VerticalSimpleDictionary(InputStream stream) throws IOException {
		this(stream, false);
	}	
	
	/**
	 * Construct a new dictionary from a stream
	 * @param stream the stream to read the data from.
	 * @param simplePOS if set to true, only the first character of the POS column will be used.
	 * @throws IOException 
	 */
	public VerticalSimpleDictionary(InputStream stream, boolean simplePOS) throws IOException {
		
		// create a buffered reader to read from
		BufferedReader myReader = new BufferedReader(
				new InputStreamReader(stream));
		
		// set instance field
		UseSimplePOS = simplePOS;
		
		// create data structures
		MyLCWords = new HashMap<String,String>();
		MyWords = new HashMap<String,String>();		
		
		// go for it!
		readDictFromReader(myReader);
	}	
	
	/**
	 * Helper method to read the dictionary 
	 * @throws IOException 
	 */
	private void readDictFromReader(BufferedReader reader) throws IOException {

		// read line by line
		int lineNumber=0;
		String line = "";
	    while (  (line = reader.readLine()) != null ) {
	    	lineNumber++;
	    	parseSingleLine(line, lineNumber);
	    } 
	}
	
	/**
	 * Helper method to read/parse a single line in the 
	 * input
	 * @throws IOException 
	 */
	private void parseSingleLine(String line, int lineNumber) throws IOException {
	     
    	// ignore comments and empty lines
        String shortline = line.trim();
        if ( (shortline.length() != 0) && (shortline.charAt(0) != '#') ) {
        	
        	String[] cols = shortline.split("\\t");
        	
        	// sanity check for empty words, empty POS are OK.
        	if ( cols.length < 1 || cols[0] == null || 
        			cols[0].compareTo("") == 0 ) {
        		throw new IOException("Dictionary read error in line "
        				+ lineNumber + "of input stream.");
        	}
        	String word = cols[0];
        	String pos = null;
        	
        	if (cols.length >= 2 ) {
        		pos = cols[1];
        	}
        	
        	if (UseSimplePOS && pos != null) {
        		pos = ""+ pos.charAt(0);
        	}
        	
        	// add entry to the data structure
        	MyLCWords.put(word.toLowerCase(), pos);
        	MyWords.put(word, pos);
        	
        }
		
	}

	/** 
	 * Helper method to search for a word
	 * @param request the request/word to search for
	 * @param lc set to true to ignore case/use lower case
	 * @return a new dictionary entry if found - or null
	 */
	private SimpleDictEntry performLookup(String request, boolean lc) {

		String pos = null;
		
		if (lc) {
			pos = MyLCWords.get(request.toLowerCase());
		} else {
			pos = MyWords.get(request);
		}
		
		if (pos == null) {
			return null;
		}
		
		return new SimpleDictEntry(request, pos);
		
	}

	/**
	 * @see de.drni.bananasplit.simpledict.SimpleDictionaryInterface#findWord(java.lang.String)
	 */
	public SimpleDictEntry findWord(String word) {
		return performLookup(word, false);

	}

	/**
	 * @see de.drni.bananasplit.simpledict.SimpleDictionaryInterface#findWordNoCase(java.lang.String)
	 */
	public SimpleDictEntry findWordNoCase(String word) {
		return performLookup(word, true);
	}

}
