/*
Copyright (C) 2005-2009 Niels Ott

Licensed under the Apache License, Version 2.0 (the "License"); you may not use 
this file except in compliance with the License. You may obtain a copy of the 
License at 
     http://www.apache.org/licenses/LICENSE-2.0 
Unless required by applicable law or agreed to in writing, software distributed 
under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
CONDITIONS OF ANY KIND, either express or implied. See the License for the specific
language governing permissions and limitations under the License.
 */

package de.drni.bananasplit.xmldict;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Properties;

import de.drni.bananasplit.simpledict.SimpleDictEntry;
import de.drni.bananasplit.simpledict.SimpleDictionaryInterface;

/** 
 * A not so simple dictionary that works similar
 * to the VerticalSimpleDictionary except for that
 * fact that data read from XML. The XML format is the
 * one used by {@link Properties}
 * The reason why XML makes sense for such a simple task is
 * that it includes character encoding information.
 * @author Niels Ott
 * @version $Id:XmlDictionary.java 24 2007-03-08 13:02:28Z niels $
 */
public class XmlDictionary implements SimpleDictionaryInterface {

	private HashMap<String,String> MyLCWords = null;
	private HashMap<String,String> MyWords = null;
	private boolean UseSimplePOS = false;	

	/**
	 * Construct a new dictionary from a file
	 * @param file the path of the file to read from
	 * @throws IOException
	 * @deprecated use {@link #XmlDictionary(File)} instead. 
	 */
	public XmlDictionary(String file) throws IOException {
		this(new File(file), false);
	}

	/**
	 * Construct a new dictionary from a file
	 * @param file the path of the file to read from
	 * @param simplePOS if set to true, only the first character of the POS column will be used.
	 * @throws IOException
	 * @deprecated use {@link #XmlDictionary(File, boolean)} instead. 
	 */
	public XmlDictionary(String file, boolean simplePOS) throws IOException {
		this(new File(file), simplePOS);
	}
	
	/**
	 * Construct a new dictionary from a file
	 * @param file the path of the file to read from
	 * @throws IOException 
	 */
	public XmlDictionary(File file) throws IOException {
		this(file, false);
	}

	/**
	 * Construct a new dictionary from a file
	 * @param file the path of the file to read from
	 * @param simplePOS if set to true, only the first character of the POS column will be used.
	 * 
	 * @throws IOException 
	 */
	public XmlDictionary(File file, boolean simplePOS) throws IOException {
		this(new FileInputStream(file), simplePOS);
	}	

	/**
	 * Construct a new dictionary from a stream
	 * @param stream the stream to read the data from.
	 * @throws IOException 
	 */
	public XmlDictionary(InputStream stream) throws IOException {
		this(stream, false);
	}	
	
	/**
	 * Construct a new dictionary from a stream
	 * @param stream the stream to read the data from.
	 * @param simplePOS if set to true, only the first character of the POS will be used.
	 * @throws IOException 
	 */
	public XmlDictionary(InputStream stream, boolean simplePOS) throws IOException {
		
		// set instance field
		UseSimplePOS = simplePOS;
		
		// create data structures
		MyLCWords = new HashMap<String,String>();
		MyWords = new HashMap<String,String>();	
		
		// read from properties file
		Properties dictProp = new Properties();
		dictProp.loadFromXML(stream);
		
		// get all of them and stuff them into our data structures
		Enumeration<?> keyEnum = dictProp.keys();
		
		while (keyEnum.hasMoreElements()) {
			String word = (String) keyEnum.nextElement();
			String pos = dictProp.getProperty(word);
			
        	if (UseSimplePOS && pos != null) {
        		pos = ""+ pos.charAt(0);
        	}
			
        	MyLCWords.put(word.toLowerCase(), pos);
        	MyWords.put(word, pos);
		}
		
	}	
	
	/** 
	 * Helper method to search for a word
	 * @param request the request/word to search for
	 * @param lc set to true to ignore case/use lower case
	 * @return a new dictionary entry if found - or null
	 */
	private SimpleDictEntry performLookup(String request, boolean lc) {

		String pos = null;
		
		if (lc) {
			pos = MyLCWords.get(request.toLowerCase());
		} else {
			pos = MyWords.get(request);
		}
		
		if (pos == null) {
			return null;
		}
		
		return new SimpleDictEntry(request, pos);
		
	}

	/**
	 * @see de.drni.bananasplit.simpledict.SimpleDictionaryInterface#findWord(java.lang.String)
	 */
	public SimpleDictEntry findWord(String word) {
		return performLookup(word, false);

	}

	/**
	 * @see de.drni.bananasplit.simpledict.SimpleDictionaryInterface#findWordNoCase(java.lang.String)
	 */
	public SimpleDictEntry findWordNoCase(String word) {
		return performLookup(word, true);
	}

}
